@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2018 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2022 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QCBORMAP_H
 #define QCBORMAP_H
@@ -56,26 +20,27 @@ class QCborContainerPrivate;
 class Q_CORE_EXPORT QCborMap
 {
 public:
-    typedef QPair<QCborValue, QCborValue> value_type;
+    typedef std::pair<QCborValue, QCborValue> value_type;
     typedef QCborValue key_type;
     typedef QCborValue mapped_type;
     typedef qsizetype size_type;
 
     class ConstIterator;
     class Iterator {
-        mutable QCborValueRef item;     // points to the value
+        QCborValueRef item {};      // points to the value
         friend class ConstIterator;
         friend class QCborMap;
         Iterator(QCborContainerPrivate *dd, qsizetype ii) : item(dd, ii) {}
     public:
         typedef std::random_access_iterator_tag iterator_category;
         typedef qsizetype difference_type;
-        typedef QPair<const QCborValueRef, QCborValueRef> value_type;
-        typedef QPair<const QCborValueRef, QCborValueRef> reference;
-        typedef QPair<const QCborValueRef, QCborValueRef> pointer;
+        typedef std::pair<QCborValueConstRef, QCborValueRef> value_type;
+        typedef std::pair<QCborValueConstRef, QCborValueRef> reference;
+        typedef std::pair<QCborValueConstRef, QCborValueRef> pointer;
 
         constexpr Iterator() = default;
         constexpr Iterator(const Iterator &) = default;
+        ~Iterator() = default;
         Iterator &operator=(const Iterator &other)
         {
             // rebind the reference
@@ -84,9 +49,16 @@ public:
             return *this;
         }
 
-        value_type operator*() const { return { {item.d, item.i - 1}, item }; }
-        QCborValueRef *operator->() const { return &item; }
-        QCborValue key() const { return QCborValueRef(item.d, item.i - 1); }
+        value_type operator*() const { return { QCborValueRef{item.d, item.i - 1}, item }; }
+        value_type operator[](qsizetype j) const { return *(*this + j); }
+        QCborValueRef *operator->() { return &item; }
+        const QCborValueConstRef *operator->() const { return &item; }
+#if QT_VERSION >= QT_VERSION_CHECK(7,0,0)
+        QCborValueConstRef
+#else
+        QCborValue
+#endif
+        key() const { return QCborValueRef(item.d, item.i - 1); }
         QCborValueRef value() const { return item; }
 
         bool operator==(const Iterator &o) const { return item.d == o.item.d && item.i == o.item.i; }
@@ -113,21 +85,23 @@ public:
     };
 
     class ConstIterator {
-        QCborValueRef item;     // points to the value
+        QCborValueConstRef item;     // points to the value
         friend class Iterator;
         friend class QCborMap;
         friend class QCborValue;
         friend class QCborValueRef;
+        constexpr ConstIterator(QCborValueConstRef it) : item{it} {}
         ConstIterator(QCborContainerPrivate *dd, qsizetype ii) : item(dd, ii) {}
     public:
         typedef std::random_access_iterator_tag iterator_category;
         typedef qsizetype difference_type;
-        typedef QPair<const QCborValueRef, const QCborValueRef> value_type;
-        typedef QPair<const QCborValueRef, const QCborValueRef> reference;
-        typedef QPair<const QCborValueRef, const QCborValueRef> pointer;
+        typedef std::pair<QCborValueConstRef, QCborValueConstRef> value_type;
+        typedef std::pair<QCborValueConstRef, QCborValueConstRef> reference;
+        typedef std::pair<QCborValueConstRef, QCborValueConstRef> pointer;
 
         constexpr ConstIterator() = default;
         constexpr ConstIterator(const ConstIterator &) = default;
+        ~ConstIterator() = default;
         ConstIterator &operator=(const ConstIterator &other)
         {
             // rebind the reference
@@ -136,10 +110,16 @@ public:
             return *this;
         }
 
-        value_type operator*() const { return { {item.d, item.i - 1}, item }; }
-        const QCborValueRef *operator->() const { return &item; }
-        QCborValue key() const { return QCborValueRef(item.d, item.i - 1); }
-        QCborValueRef value() const { return item; }
+        value_type operator*() const { return { QCborValueRef(item.d, item.i - 1), item }; }
+        value_type operator[](qsizetype j) const { return *(*this + j); }
+        const QCborValueConstRef *operator->() const { return &item; }
+#if QT_VERSION >= QT_VERSION_CHECK(7,0,0)
+        QCborValueConstRef
+#else
+        QCborValue
+#endif
+        key() const { return QCborValueRef(item.d, item.i - 1); }
+        QCborValueConstRef value() const { return item; }
 
         bool operator==(const Iterator &o) const { return item.d == o.item.d && item.i == o.item.i; }
         bool operator!=(const Iterator &o) const { return !(*this == o); }
@@ -159,8 +139,8 @@ public:
         ConstIterator operator--(int) { ConstIterator n = *this; item.i -= 2; return n; }
         ConstIterator &operator+=(qsizetype j) { item.i += 2 * j; return *this; }
         ConstIterator &operator-=(qsizetype j) { item.i -= 2 * j; return *this; }
-        ConstIterator operator+(qsizetype j) const { return ConstIterator({ item.d, item.i + 2 * j }); }
-        ConstIterator operator-(qsizetype j) const { return ConstIterator({ item.d, item.i - 2 * j }); }
+        ConstIterator operator+(qsizetype j) const { return ConstIterator{ item.d, item.i + 2 * j }; }
+        ConstIterator operator-(qsizetype j) const { return ConstIterator{ item.d, item.i - 2 * j }; }
         qsizetype operator-(ConstIterator j) const { return (item.i - j.item.i) / 2; }
     };
 
@@ -190,7 +170,7 @@ public:
 
     QCborValue value(qint64 key) const
     { const_iterator it = find(key); return it == end() ? QCborValue() : it.value(); }
-    QCborValue value(QLatin1String key) const
+    QCborValue value(QLatin1StringView key) const
     { const_iterator it = find(key); return it == end() ? QCborValue() : it.value(); }
     QCborValue value(const QString & key) const
     { const_iterator it = find(key); return it == end() ? QCborValue() : it.value(); }
@@ -202,7 +182,7 @@ public:
 #endif
     const QCborValue operator[](qint64 key) const
     { const_iterator it = find(key); return it == end() ? QCborValue() : it.value(); }
-    const QCborValue operator[](QLatin1String key) const
+    const QCborValue operator[](QLatin1StringView key) const
     { const_iterator it = find(key); return it == end() ? QCborValue() : it.value(); }
     const QCborValue operator[](const QString & key) const
     { const_iterator it = find(key); return it == end() ? QCborValue() : it.value(); }
@@ -213,13 +193,13 @@ public:
     { return operator[](QString::fromUtf8(key, N - 1)); }
 #endif
     QCborValueRef operator[](qint64 key);
-    QCborValueRef operator[](QLatin1String key);
+    QCborValueRef operator[](QLatin1StringView key);
     QCborValueRef operator[](const QString & key);
     QCborValueRef operator[](const QCborValue &key);
 
     QCborValue take(qint64 key)
     { const_iterator it = constFind(key); if (it != constEnd()) return extract(it); return QCborValue(); }
-    QCborValue take(QLatin1String key)
+    QCborValue take(QLatin1StringView key)
     { const_iterator it = constFind(key); if (it != constEnd()) return extract(it); return QCborValue(); }
     QCborValue take(const QString &key)
     { const_iterator it = constFind(key); if (it != constEnd()) return extract(it); return QCborValue(); }
@@ -227,7 +207,7 @@ public:
     { const_iterator it = constFind(key); if (it != constEnd()) return extract(it); return QCborValue(); }
     void remove(qint64 key)
     { const_iterator it = constFind(key); if (it != constEnd()) erase(it); }
-    void remove(QLatin1String key)
+    void remove(QLatin1StringView key)
     { const_iterator it = constFind(key); if (it != constEnd()) erase(it); }
     void remove(const QString & key)
     { const_iterator it = constFind(key); if (it != constEnd()) erase(it); }
@@ -235,7 +215,7 @@ public:
     { const_iterator it = constFind(key); if (it != constEnd()) erase(it); }
     bool contains(qint64 key) const
     { const_iterator it = find(key); return it != end(); }
-    bool contains(QLatin1String key) const
+    bool contains(QLatin1StringView key) const
     { const_iterator it = find(key); return it != end(); }
     bool contains(const QString & key) const
     { const_iterator it = find(key); return it != end(); }
@@ -277,15 +257,15 @@ public:
     bool empty() const { return isEmpty(); }
 
     iterator find(qint64 key);
-    iterator find(QLatin1String key);
+    iterator find(QLatin1StringView key);
     iterator find(const QString & key);
     iterator find(const QCborValue &key);
     const_iterator constFind(qint64 key) const;
-    const_iterator constFind(QLatin1String key) const;
+    const_iterator constFind(QLatin1StringView key) const;
     const_iterator constFind(const QString & key) const;
     const_iterator constFind(const QCborValue &key) const;
     const_iterator find(qint64 key) const { return constFind(key); }
-    const_iterator find(QLatin1String key) const { return constFind(key); }
+    const_iterator find(QLatin1StringView key) const { return constFind(key); }
     const_iterator find(const QString & key) const { return constFind(key); }
     const_iterator find(const QCborValue &key) const { return constFind(key); }
 
@@ -295,7 +275,7 @@ public:
         v = value_;
         return { d.data(), v.i };
     }
-    iterator insert(QLatin1String key, const QCborValue &value_)
+    iterator insert(QLatin1StringView key, const QCborValue &value_)
     {
         QCborValueRef v = operator[](key);  // detaches
         v = value_;
@@ -324,6 +304,7 @@ public:
     QJsonObject toJsonObject() const;
 
 private:
+    friend class QCborContainerPrivate;
     friend class QCborValue;
     friend class QCborValueRef;
     friend class QJsonPrivate::Variant;
@@ -340,6 +321,7 @@ inline QCborValue::QCborValue(QCborMap &&m)
 {
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(7, 0, 0) && !defined(QT_BOOTSTRAPPED)
 inline QCborMap QCborValueRef::toMap() const
 {
     return concrete().toMap();
@@ -349,6 +331,17 @@ inline QCborMap QCborValueRef::toMap(const QCborMap &m) const
 {
     return concrete().toMap(m);
 }
+#endif
+
+inline QCborMap QCborValueConstRef::toMap() const
+{
+    return concrete().toMap();
+}
+
+inline QCborMap QCborValueConstRef::toMap(const QCborMap &m) const
+{
+    return concrete().toMap(m);
+}
 
 Q_CORE_EXPORT size_t qHash(const QCborMap &map, size_t seed = 0);
 
